module cartridge()
{
	$fn = 50;

	/* Rounding/fillet radius and additional margin of subtracted
	material. The additional margin helps avoid geometry
	problems. */

	rr = 1;
	ro = rr;
	extra = 0.1;

	module fillet(r, h) {
		translate([0, 0, -h/2])
			difference() {
				cube([r + extra, r + extra, h]);
				cylinder(r = r, h = h);
			}
	}

	/* Cartridge dimensions. */

	width = 89.0;
	height = 68.5;
	depth = 16.5;

	/* Details. */

	front_label_width = 83.0;
	front_label_height = 46.0;
	front_label_depth = 1.0;
	front_label_offset_from_bottom = 19.5;

	top_label_width = front_label_width;
	top_label_height = 11.5;
	top_label_depth = front_label_depth;
	top_label_offset_from_front = 2.5;

	groove_depth = 1.0;

	back_cavity_width = 68.0;
	back_cavity_inner_width = 65.0;
	back_cavity_offset_from_left = 10.5;
	back_cavity_inner_offset_from_left = 12.0;
	back_cavity_height = 13.5;
	back_cavity_inner_height = 12.0;
	back_cavity_depth = 1.5;

	inner_back_cavity_offset = 1.0;
	inner_back_cavity_offset_from_left = 2.0;
	inner_back_slope_offset_from_left = 10.0;
	inner_back_slope_width = 2.5;
	inner_back_slope_depth = 2.5;

	inner_back_edge_offset = 2.0;
	inner_back_edge_width = 69.0;
	inner_back_edge_height = 3.0;
	inner_back_edge_depth = 1.5;

	inner_front_edge_offset = 0.5;
	inner_front_edge_width = 87.0;
	inner_front_edge_height = 3.0;
	inner_front_edge_depth = 1.5;

	bottom_from_base = 13.5;

	/* Side thicknesses. */

	front = 2;
	back = 3.5;
	top = 3;
	left = 2;
	right = 2;
	bottom = 2;

	difference() {

		/* The cartridge surfaces. */

		union() {
			translate([0, -depth / 2 + front / 2, 0])
				cube([width, front, height], center = true);
			translate([0, depth / 2 - back / 2, 0])
				cube([width, back, height], center = true);
			translate([-width / 2 + left / 2, 0, 0])
				cube([left, depth, height], center = true);
			translate([width / 2 - right / 2, 0, 0])
				cube([right, depth, height], center = true);
			translate([0, 0, height / 2 - top / 2])
				cube([width, depth, top], center = true);
			translate([0, 0, -height / 2 + bottom / 2 + bottom_from_base])
				cube([width, depth, top], center = true);
		}

		/* Label insets. */

		union() {

			/* Front label. */

			translate([-front_label_width / 2, -depth / 2,
				front_label_offset_from_bottom - height / 2])
				cube([front_label_width, front_label_depth,
					front_label_height]);

			/* Top label. */

			translate([-top_label_width / 2,
				-depth / 2 + top_label_offset_from_front,
				height / 2 - top_label_depth])
				cube([top_label_width, top_label_height,
					top_label_depth]);
		}

		/* Top and side grooves. */

		union() {

			/* Left groove. */

			translate([-width / 2 + groove_depth / 2, 0, 0])
				cube([groove_depth, groove_depth, height],
					center = true);

			/* Right groove. */

			translate([width / 2 - groove_depth / 2, 0, 0])
				cube([groove_depth, groove_depth, height],
					center = true);

			/* Top groove. */

			translate([0, 0, height / 2 - groove_depth / 2])
				cube([width, groove_depth, groove_depth],
					center = true);
		}

		/* Back cavity. */

		intersection() {

			/* From the bottom upwards. */

			translate([0, depth / 2, -height / 2])
				linear_extrude(height = back_cavity_height)
					translate([-width / 2, 0, 0])
						polygon([
							[back_cavity_offset_from_left, 0],
							[back_cavity_inner_offset_from_left,
								-back_cavity_depth],
							[back_cavity_inner_offset_from_left +
								back_cavity_inner_width,
								-back_cavity_depth],
							[back_cavity_offset_from_left +
								back_cavity_width, 0]
							]);

			/* From left to right. */

			translate([back_cavity_width / 2, depth / 2, -height / 2])
				rotate([0, -90, 0])
					linear_extrude(height = back_cavity_width)
						polygon([
							[-extra, -back_cavity_depth],
							[back_cavity_inner_height,
								-back_cavity_depth],
							[back_cavity_height, 0],
							[-extra, 0]
							]);
		}

		/* Inner back cavities. */

		translate([0, depth / 2 - inner_back_cavity_offset, -height / 2])
			linear_extrude(height = bottom_from_base)
				translate([-width / 2, 0, 0])
					polygon([
						[inner_back_cavity_offset_from_left, 0],
						[inner_back_slope_offset_from_left, 0],
						[inner_back_slope_offset_from_left +
							inner_back_slope_width,
							-inner_back_slope_depth],
						[inner_back_cavity_offset_from_left,
							-inner_back_slope_depth]
						]);

		translate([0, depth / 2 - inner_back_cavity_offset, -height / 2])
			linear_extrude(height = bottom_from_base)
				translate([-width / 2, 0, 0])
					polygon([
						[width - inner_back_slope_offset_from_left, 0],
						[width - inner_back_cavity_offset_from_left, 0],
						[width - inner_back_cavity_offset_from_left,
							-inner_back_slope_depth],
						[width - inner_back_slope_offset_from_left -
							inner_back_slope_width,
							-inner_back_slope_depth]
						]);

		/* Inner back edge cavity. */

		translate([inner_back_edge_width / 2,
			depth / 2 - inner_back_edge_offset, -height / 2])
			rotate([0, -90, 0])
				linear_extrude(height = inner_back_edge_width)
					polygon([
						[-extra, -inner_back_edge_depth],
						[inner_back_edge_height, -inner_back_edge_depth],
						[0, 0],
						[-extra, 0]
						]);

		/* Inner front edge cavity. */

		translate([inner_front_edge_width / 2,
			-depth / 2 + inner_front_edge_offset, -height / 2])
			rotate([0, -90, 0])
				linear_extrude(height = inner_front_edge_width)
					polygon([
						[-extra, 0],
						[0, 0],
						[inner_front_edge_height, inner_front_edge_depth],
						[-extra, inner_front_edge_depth],
						]);

		/* Fillets to round off the edges. */

		union() {

			/* Top left and right rounding. */
		
			translate([-width / 2 + ro, 0, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([90, 0, 0])
						fillet(rr, depth);
			translate([width / 2 - ro, 0, height / 2 - ro])
				rotate([90, 0, 0])
					fillet(rr, depth);
		
			/* Top back and front rounding. */
		
			translate([0, depth / 2 - ro, height / 2 - ro])
				rotate([0, -90, 0])
					fillet(rr, width);
			translate([0, -depth / 2 + ro, height / 2 - ro])
				rotate([0, 0, 180])
					rotate([0, -90, 0])
						fillet(rr, width);
		
			/* Edge rounding. */
		
			translate([width / 2 - ro, depth / 2 - ro, 0])
				fillet(rr, height);
			translate([-width / 2 + ro, depth / 2 - ro, 0])
				rotate([0, 0, 90])
					fillet(rr, height);
			translate([-width / 2 + ro, -depth / 2 + ro, 0])
				rotate([0, 0, 180])
					fillet(rr, height);
			translate([width / 2 - ro, -depth / 2 + ro, 0])
				rotate([0, 0, 270])
					fillet(rr, height);
		}
	}
}

cartridge();

